//******************************************************************************
//  Life.java:  Applet
//      (c) 1997 Josef Pelikan, KSVI MFF UK Praha
//          Josef.Pelikan@mff.cuni.cz, http://sun3.ms.mff.cuni.cz/~pepca/
//******************************************************************************

import java.applet.*;
import java.awt.*;

//==============================================================================
//  Main class for applet life
//==============================================================================

public class Life extends Applet implements Runnable
{
        // animation is not running:
    boolean stopped = true;

        // animation thread:
    Thread animThread = null;

        // applet parameters:
    private int cellSize = 5;           // cell size in pixels
    private int colors = 12;            // cells will grow old
    private int delay = 200;            // animation delay in ms

    final int DDelay = 10;              // delta delay

        // parameter identifiers:
    private final String PARAMcellSize = "cellsize";
    private final String PARAMcolors = "colors";
    private final String PARAMdelay = "delay";

    final int ESC = 0x1B;               // ESC key

        // cell color scale:
    final Color hotColor = new Color(255,128,64);
    final Color coldColor = new Color(32,96,255);   // 64,0,255


    public String getAppletInfo()
    {
        return "Name: Life simulation\r\n" +
               "Author: Pepca (Josef.Pelikan@mff.cuni.cz)\r\n";
    }   // Life.getAppletInfo


    public String[][] getParameterInfo()
    {
        String[][] info =
        {
            { PARAMcellSize, "int", "Cell size in pixels" },
            { PARAMcolors, "int", "Maximum cell color (according to the age)" },
            { PARAMdelay, "int", "Animation delay in miliseconds" },
        };
        return info;        
    }   // Life.getParameterInfo


        // GUI objects:
    private LifeCanvas canvas;                  // playfield

    private Button stepButton, clearButton;     // one step button, clear button
    private Button leftButton, rightButton,     // shift buttons
                   upButton, downButton;
    private Button stopButton, slowerButton,    // animation buttons
                   fasterButton;
    private Button zoomInButton, zoomOutButton; // zoom buttons
    private Checkbox gridBox;                   // grid chackbox
    private Checkbox agesBox;                   // color ages checkbox


    public void init ()
    {
            // applet parameters parsing:
        String param;

        param = getParameter(PARAMcellSize);
        try {
            if ( param != null )
                cellSize = Integer.parseInt(param);
            } catch ( Exception e ) {}

        param = getParameter(PARAMcolors);
        try {
            if ( param != null )
                colors = Integer.parseInt(param);
            } catch ( Exception e ) {}

        param = getParameter(PARAMdelay);
        try {
            if ( param != null )
                delay = Integer.parseInt(param);
            } catch ( Exception e ) {}

            // playfield initialization:
        Rectangle myBounds = bounds();
        canvas = new LifeCanvas( new Dimension(myBounds.width,myBounds.height-72), cellSize,
                                 colors, LifeCanvas.ciHSBinc, hotColor, coldColor );

            // buttons:
        stepButton = new Button("Step");
        clearButton = new Button("Clear");

        stopButton = new Button("Start");
        slowerButton = new Button("Slower");
        fasterButton = new Button("Faster");

        leftButton = new Button("Left");
        rightButton = new Button("Right");
        upButton = new Button("Up");
        downButton = new Button("Down");
        zoomInButton = new Button("Zoom in");
        zoomOutButton = new Button("Zoom out");
        gridBox = new Checkbox("grid");
        agesBox = new Checkbox("aging",null,true);

        setBackground(new Color(255,255,255));
        
            // add all components:
        add(stepButton);
        add(clearButton);

        add(new Label("Animate:"));
        add(stopButton);
        add(slowerButton);
        add(fasterButton);

        Panel displayPanel = new Panel();
        displayPanel.setBackground(new Color(255,255,255));
        displayPanel.add(new Label("View:"));
        displayPanel.add(leftButton);
        displayPanel.add(rightButton);
        displayPanel.add(upButton);
        displayPanel.add(downButton);
        displayPanel.add(zoomInButton);
        displayPanel.add(zoomOutButton);
        displayPanel.add(new Label(""));
        displayPanel.add(gridBox);
        displayPanel.add(agesBox);
        add(displayPanel);

        add(canvas);
            // validate components:
        validate();
        canvas.fixSize();
    }   // Life.init


    public void startAnimation ()
    {
        stopped = false;
        animThread = new Thread(this,"AnimationThread");
        animThread.start();         // runs Life.run() method
    }   // Life.start


    private long waiting = 0L;      // actual wait time


    public void run ()
    {
        if ( Thread.currentThread() != animThread )     // repaint thread:
            return;
            // animation thread:
        Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
        do {
            try {
                    // remember the starting time:
                long stopTime = System.currentTimeMillis() + delay;
                    // advance the animation:
                canvas.nextGeneration();
                    // repaint the canvas:
                canvas.setTimes(delay,waiting);
                canvas.repaintMe(false);
                    // animation delay:
                waiting = stopTime - System.currentTimeMillis();
                if ( waiting > 0L ) Thread.sleep(waiting);
                else                waiting = 0L;
                }
            catch ( InterruptedException e )
                {}
            } while ( !stopped );       // no preemption!
    }   // Life.run


    private void startStop ()
    {
        if ( stopped ) {
            stopButton.setLabel("Stop");
            stepButton.disable();
            clearButton.disable();
            startAnimation();
            return;
            }
        stopButton.setLabel("Start");
        stepButton.enable();
        clearButton.enable();
        stopped = true;                 // no preemption!
    }   // Life.run


    public boolean action ( Event e, Object arg )
    {
        Object target = e.target;
         
        if ( target == stepButton ) {       // Step button
            long startTime = System.currentTimeMillis();
                // advance one step:
            canvas.nextGeneration();
                // repaint the canvas:
            canvas.setTimes(0L,System.currentTimeMillis()-startTime);
            canvas.repaintMe(true);
            return true;
            }

        if ( target == clearButton ) {      // Clear button
            canvas.clear();
            canvas.repaintMe(true);
            return true;
            }

        if ( target == stopButton ) {       // Stop/start button
            startStop();
            return true;
            }

        if ( target == slowerButton ) {     // Slow down
            delay += DDelay;
            canvas.setTimes(delay,waiting);
            canvas.repaintMe(true);
            return true;
            }

        if ( target == fasterButton ) {     // Speed up
            if ( delay > DDelay ) {
                delay -= DDelay;
                canvas.setTimes(delay,waiting);
                canvas.repaintMe(true);
                }
            return true;
            }

        if ( target == leftButton ) {       // shift left
            canvas.shift(-1,0);
            return true;
            }

        if ( target == rightButton ) {      // shift right
            canvas.shift(1,0);
            return true;
            }

        if ( target == upButton ) {         // shift up
            canvas.shift(0,-1);
            return true;
            }

        if ( target == downButton ) {       // shift down
            canvas.shift(0,1);
            return true;
            }

        if ( target == zoomInButton ) {     // zoom in
            canvas.zoomInOut(1);
            return true;
            }

        if ( target == zoomOutButton ) {    // zoom out
            canvas.zoomInOut(-1);
            return true;
            }

        if ( target == gridBox ) {          // toggle grid
            canvas.setBoxGrid(gridBox.getState());
            canvas.repaintAll();
            return true;
            }

        if ( target == agesBox ) {          // toggle cell ages
            canvas.setAges(agesBox.getState());
            canvas.repaintAll();
            return true;
            }

        return false;
    }   // Life.action


    public boolean keyDown ( Event evt, int key )
    {
        if ( super.keyDown(evt,key) ) return true;
        if ( key == ESC && stopped == false ) {
            startStop();
            return true;
            }
        return canvas.keyDown(evt,key);
    }   // Life.keyDown

}   // Life
